﻿//-----------------------------------------------------------------------------
//	シリアルポートを使った Nucleo-F446 のコントロールの例
//
//		マイコン側に "ON" または "OFF" を送る
//
//		マイコン側のプログラム： TrG_SerialComLed
//		ボーレート：9600 baud
//
//		作成者：三上直樹，2021/08/01 作成，©三上直樹 2021
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.IO.Ports;
using Mikami.ComPort;

using ThreadTimer = System.Threading.Timer;

namespace CQ_SerialComLed
{
	public partial class Form1 : Form
	{
		// マイコン側のプログラムを識別するための ID
		private const string DEVICE_ID_ = "LED_CTRLer";
		// シリアルポートと関連して使うクラスのオブジェクト
		private readonly MySerialPort sp_;
		// マイコンに接続されている COM ポートのリスト作成
		private readonly Queue<string> port_ = ComPort.List();

		private string rxData_ = "";	// 受信データ用バッファ
		private ThreadTimer timerAck_;	// マイコンからの応答を待つ際に使う

		public Form1()
		{
			InitializeComponent();

			if (port_.Count == 0)
			{
				MessageBox.Show("COM ポートにマイコンが接続されていないので終了します．");
				Environment.Exit(0);
			}
			sp_ = new MySerialPort(serialPort1);
			// ComboBox の Items の内容を作成
			comboBox1.Items.AddRange(port_.OrderBy(x => x).ToArray());
			comboBox1.SelectedIndex = 0;
		}

		// "接続" ボタンのクリックに対応
		private void button1_Click(object sender, EventArgs e)
		{
			if (!sp_.Open(comboBox1.Text)) return;

			sp_.SendTo(DEVICE_ID_);		// マイコンへ "ACK" の送信を要求
			timerAck_ = new ThreadTimer(AckTimeout, null, 100, 0);
		}

		// 点滅処理を有効にするための指令を送信する
		private void radioButton1_Click(object sender, EventArgs e)
		{	sp_.SendTo("ON");	}

		// 点滅処理を無効にするための指令を送信する
		private void radioButton2_Click(object sender, EventArgs e)
		{	sp_.SendTo("OFF");	}

		// 終了時に シリアルポートがオープンであればクローズする
		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			try { if (serialPort1.IsOpen) serialPort1.Close(); }
			catch (Exception ex) { sp_.ExceptionCatch(ex); }
		}

		// データ受信イベントに対応するハンドラ
		private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
			try {
				rxData_ = serialPort1.ReadLine();
				if (rxData_  == "ACK")
				{
					timerAck_.Change(-1, 0);
					toolStripLabel1.Text = "マイコン側の準備ができました．";
					toolStripLabel2.Text = " " + serialPort1.PortName + " ";
					Invoke(() => button1.Enabled = false);
					Invoke(() => panel1.Enabled = true);
				}
				else
					Reply();	// "ACK" 受信以外の処理
			}
			catch (Exception ex) { sp_.ExceptionCatch(ex); }
		}

		// 送信したデータに対する応答
		private void Reply()
			=> Invoke(() => label1.Text = "LED 点滅： " + rxData_);

		// マイコンの準備ができていない（"ACK" が返ってこない）場合
		private void AckTimeout(object state)
		{
			toolStripLabel1.Text = "マイコン側の準備ができていません．";
			if (serialPort1.IsOpen) serialPort1.Close();
		}

		//--------------------------------------------------------------
		// 別のスレッドを使う場合に使用
		private void Invoke(Action action) => base.Invoke(action);
	}
}
